/*         oled     */
/*     SSD1306   I2C   */
/*    .  http://ad-res.ru/controllers/   */


#ifndef __SSD1306_I2C_C__
#define __SSD1306_I2C_C__


#include "stm32f10x_gpio.h"
#include "Font_Verter_10x15.h"



#define I2C_PORT				GPIOB
#define I2C_SCL_PIN				10
#define I2C_SDA_PIN				11
//      I2C_Init() . 

#define SSD1306_ADDR			0x78

#ifndef SSD1306_WIDTH
#define SSD1306_WIDTH			128
#endif
#ifndef SSD1306_HEIGHT
#define SSD1306_HEIGHT			64
#endif

//     SCL.   .   6  SCL=330 
#define I2CSWM_DELAY			6


volatile uint8_t sMessage[16] = {0};

volatile uint8_t SSD1306_Buffer[SSD1306_WIDTH * SSD1306_HEIGHT / 8] = {0};



#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define I2CSWM_SETSCL() 	(I2C_PORT->BSRR |= 1<<I2C_SCL_PIN)
#define I2CSWM_CLEARSCL() 	(I2C_PORT->BSRR |= 1<<16<<I2C_SCL_PIN)
#define I2CSWM_GETSCL() 	((I2C_PORT->IDR & (1<<I2C_SCL_PIN)) == 0 ? 0 : 1 )

#define I2CSWM_SETSDA() 	(I2C_PORT->BSRR |= 1<<I2C_SDA_PIN)
#define I2CSWM_CLEARSDA() 	(I2C_PORT->BSRR |= 1<<16<<I2C_SDA_PIN)
#define I2CSWM_GETSDA() 	((I2C_PORT->IDR & (1<<I2C_SDA_PIN)) == 0 ? 0 : 1 )

#define I2CSWM_DIRECTION_TX 	0
#define I2CSWM_DIRECTION_RX 	1

#define I2CSWM_NACK 			1
#define I2CSWM_ACK				0



#define SSD1306_COMMAND			0x00
#define SSD1306_DATA			0xC0
#define SSD1306_DATA_CONTINUE	0x40

#define RST_NOT_IN_USE	255

// SSD1306 Commandset
// ------------------
// Fundamental Commands
#define SSD1306_SET_CONTRAST_CONTROL					0x81
#define SSD1306_DISPLAY_ALL_ON_RESUME					0xA4
#define SSD1306_DISPLAY_ALL_ON							0xA5
#define SSD1306_NORMAL_DISPLAY							0xA6
#define SSD1306_INVERT_DISPLAY							0xA7
#define SSD1306_DISPLAY_OFF								0xAE
#define SSD1306_DISPLAY_ON								0xAF
#define SSD1306_NOP										0xE3
// Scrolling Commands
#define SSD1306_HORIZONTAL_SCROLL_RIGHT					0x26
#define SSD1306_HORIZONTAL_SCROLL_LEFT					0x27
#define SSD1306_HORIZONTAL_SCROLL_VERTICAL_AND_RIGHT	0x29
#define SSD1306_HORIZONTAL_SCROLL_VERTICAL_AND_LEFT		0x2A
#define SSD1306_DEACTIVATE_SCROLL						0x2E
#define SSD1306_ACTIVATE_SCROLL							0x2F
#define SSD1306_SET_VERTICAL_SCROLL_AREA				0xA3
// Addressing Setting Commands
#define SSD1306_SET_LOWER_COLUMN						0x00
#define SSD1306_SET_HIGHER_COLUMN						0x10
#define SSD1306_MEMORY_ADDR_MODE						0x20
#define SSD1306_SET_COLUMN_ADDR							0x21
#define SSD1306_SET_PAGE_ADDR							0x22
// Hardware Configuration Commands
#define SSD1306_SET_START_LINE							0x40
#define SSD1306_SET_SEGMENT_REMAP						0xA0
#define SSD1306_SET_MULTIPLEX_RATIO						0xA8
#define SSD1306_COM_SCAN_DIR_INC						0xC0
#define SSD1306_COM_SCAN_DIR_DEC						0xC8
#define SSD1306_SET_DISPLAY_OFFSET						0xD3
#define SSD1306_SET_COM_PINS							0xDA
#define SSD1306_CHARGE_PUMP								0x8D
// Timing & Driving Scheme Setting Commands
#define SSD1306_SET_DISPLAY_CLOCK_DIV_RATIO				0xD5
#define SSD1306_SET_PRECHARGE_PERIOD					0xD9
#define SSD1306_SET_VCOM_DESELECT						0xDB






inline static void I2C_Delay( int n ) { while( n-- ); }
void SendOneCommand(uint8_t command);

unsigned long rand(void);
int abs(int i);


void Delay(volatile uint32_t nCount) {
	for(nCount *= 100000; nCount; nCount--);
}




void I2C_Init(void) {
	GPIO_InitTypeDef gpio;
	gpio.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
	gpio.GPIO_Mode = GPIO_Mode_Out_OD;
	gpio.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(I2C_PORT, &gpio);

    I2CSWM_SETSDA();
    I2CSWM_SETSCL();

	SendOneCommand (0xA8);		// 64 
	SendOneCommand (0x3F);
	SendOneCommand (0x20);		//  
	SendOneCommand (0x00);
	SendOneCommand (0xD3);		// Set Display Offset
	SendOneCommand (0);
	SendOneCommand (0xA1);		//   (127...0)
	SendOneCommand (0xC8);		//   (63...0)
	SendOneCommand (0x81);		// 
	SendOneCommand (0x60);
	SendOneCommand (0x8D);		// ON display
	SendOneCommand (0x14);
	SendOneCommand (0xAF);
	SendOneCommand (0x21);		// 0  , 127 
	SendOneCommand (0);
	SendOneCommand (127);
	SendOneCommand (0x22);		// 0  , 7 
	SendOneCommand (0);
	SendOneCommand (7);

	I2C_Delay(2500);
}



inline static void I2C_Start() {
    I2CSWM_SETSDA();
    I2CSWM_SETSCL();
    I2C_Delay(I2CSWM_DELAY);
    I2CSWM_CLEARSDA();
    I2C_Delay(I2CSWM_DELAY);
    I2CSWM_CLEARSCL();
    I2C_Delay(I2CSWM_DELAY);
}




inline static void I2C_Stop() {
    I2CSWM_CLEARSCL();
    I2CSWM_CLEARSDA();
    I2C_Delay(I2CSWM_DELAY);
    I2CSWM_SETSCL();
    I2C_Delay(I2CSWM_DELAY);
    I2CSWM_SETSDA();
    I2C_Delay(I2CSWM_DELAY);
}




inline static uint_fast8_t I2C_Write_Byte(uint8_t b) {
    uint_fast8_t db = 0x80, ack;

    for(uint_fast8_t i = 0; i < 8; i++) {
		I2C_Delay(I2CSWM_DELAY / 2);
        if( b & db ) {
            I2CSWM_SETSDA();
			}
		else {
            I2CSWM_CLEARSDA();
			}
		I2CSWM_SETSCL();
		I2C_Delay(I2CSWM_DELAY);
		I2CSWM_CLEARSCL();
		I2C_Delay(I2CSWM_DELAY / 2);
        db >>= 1;
		}

    /*  ACK */
    I2CSWM_SETSDA(); 				//    SDA 
	I2C_Delay(I2CSWM_DELAY / 2);
    I2CSWM_SETSCL(); 				//   
	I2C_Delay(I2CSWM_DELAY);
	ack = I2CSWM_GETSDA();
	I2CSWM_CLEARSCL();
	I2C_Delay(I2CSWM_DELAY / 2);

    return ack;
}





uint8_t SendCommand(uint8_t command, uint8_t params, uint8_t param1, uint8_t param2) {
	I2C_Start();
	// 0x78 = 01111000    - Rx/Tx,  - D/C,     
	if(I2C_Write_Byte(SSD1306_ADDR)) 		return FALSE;
	// 0x80 = 10000000   - isCommand,  - D/C,  - ,     
	if(I2C_Write_Byte(SSD1306_COMMAND)) 	return FALSE;
	if(I2C_Write_Byte(command)) 			return FALSE;
	if(params > 0) {
		if(I2C_Write_Byte(SSD1306_ADDR)) 	return FALSE;
		if(I2C_Write_Byte(SSD1306_COMMAND)) return FALSE;
		if(I2C_Write_Byte(param1))			return FALSE;
		}
	if(params > 1) {
		if(I2C_Write_Byte(SSD1306_ADDR)) 	return FALSE;
		if(I2C_Write_Byte(SSD1306_COMMAND)) return FALSE;
		if(I2C_Write_Byte(param2)) 			return FALSE;
		}
	I2C_Stop();

	return TRUE;
}



void SendOneCommand(uint8_t command) {
	SendCommand(command, 0, 0, 0);
}


void UpdateScreen(void) {
	int i, m;

	for (m = 0; m < 8; m++) {
		SendCommand((0xB0 + m), 0, 0, 0);
		SendCommand(0x00, 0, 0, 0);
		SendCommand(0x10, 0, 0, 0);

    	I2C_Start();
    	I2C_Write_Byte(SSD1306_ADDR);
    	I2C_Write_Byte(SSD1306_DATA_CONTINUE);
		for(i=0; i<128; i++) {
			I2C_Write_Byte(*(SSD1306_Buffer + SSD1306_WIDTH * m + i));
			}
    	I2C_Stop();
	}
}




void UpdateScreenBottom(void) {
	int i, m;

	for (m = 4; m < 8; m++) {
		SendCommand((0xB0 + m), 0, 0, 0);
		SendCommand(0x00, 0, 0, 0);
		SendCommand(0x10, 0, 0, 0);

    	I2C_Start();
    	I2C_Write_Byte(SSD1306_ADDR);
    	I2C_Write_Byte(SSD1306_DATA_CONTINUE);
		for(i=0; i<128; i++) {
			I2C_Write_Byte(*(SSD1306_Buffer + SSD1306_WIDTH * m + i));
			}
    	I2C_Stop();
	}
}




void SetPixel(uint8_t x, uint8_t y, uint8_t color) {
	uint8_t yy, the_byte;
	yy = y % (SSD1306_HEIGHT / 8);

	the_byte = *(SSD1306_Buffer + SSD1306_WIDTH * (y/8) + x);
	if(color)
		the_byte |=  (1 << yy);
	else
		the_byte &= ~(1 << yy);

	*(SSD1306_Buffer + SSD1306_WIDTH * (y/8) + x) = the_byte;
}



//     
void DrawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t color) {
	const int deltaX = abs(x2 - x1);
	const int deltaY = abs(y2 - y1);
	const int signX = x1 < x2 ? 1 : -1;
	const int signY = y1 < y2 ? 1 : -1;
	int error = deltaX - deltaY, error2;

	SetPixel(x2, y2, color);
	while(x1 != x2 || y1 != y2) {
		SetPixel(x1, y1, color);
		error2 = error * 2;
		if(error2 > -deltaY) {
			error -= deltaY;
			x1 += signX;
			}
		if(error2 < deltaX) {
			error += deltaX;
			y1 += signY;
			}
		}
}




//     
//     (0, 0)  (x, y)
int16_t SquareDist(int16_t x, int16_t y) {
	return (x * x + y * y);
}



//     
void DrawCircle(int16_t x0, int16_t y0, int16_t r, uint8_t color) {
	int16_t x = 0, y = r; //   
	int16_t move[3]; //    3 ,      

	while(y >= (r >> 1)) {
		SetPixel(x0 + x, y0 + y, color);
		SetPixel(x0 - x, y0 + y, color);
		SetPixel(x0 + x, y0 - y, color);
		SetPixel(x0 - x, y0 - y, color);
		SetPixel(x0 + y, y0 + x, color);
		SetPixel(x0 - y, y0 + x, color);
		SetPixel(x0 + y, y0 - x, color);
		SetPixel(x0 - y, y0 - x, color);

		move[0] = abs(SquareDist(x + 1, y) - r * r);
		move[1] = abs(SquareDist(x, y - 1) - r * r);
		move[2] = abs(SquareDist(x + 1, y - 1) - r * r);

		if(move[0] <= move[1] && move[0] <= move[2]) { x++; }
		else
		if(move[1] <= move[0] && move[1] <= move[2]) { y--; }
		else
		if(move[2] <= move[0] && move[2] <= move[1]) { x++; y--; }
		}
}




void PutChar(uint8_t x, uint8_t y, uint8_t symbol, uint8_t color) {
	uint8_t i, j, k, chl, h, fnt_count;

	h = 0; //  h -      
	fnt_count = sizeof(fnt_Verter) / 21;
	for(i=0; i < fnt_count; i++) {
		if(fnt_Verter[i][0] == symbol) {
			h = i;
			break;
			}
		}
	if(!h) return;
	i = h;

	h = 1; //  h - :    /   
	for(j=0; j<20; j++) {
		chl = fnt_Verter[i][j+1];
		if(h) {
			for(k=0; k<8; k++) {
				SetPixel(x + (j/2), y + 7*h + k, (chl >> k) & 0x01);
				}
			h = 0;
			}
		else {
			for(k=1; k<8; k++) {
				SetPixel(x + (j/2), y + 7*h + k-1, (chl >> k) & 0x01);
				}
			h = 1;
			}
		}
}




void PutString(uint8_t x, uint8_t y, volatile uint8_t *Mess, uint8_t color) {
	uint8_t i, max, interval = 10;

	max = sizeof(sMessage);
	for(i=0; *(Mess + i) && i < max; i++) {
		PutChar(x + interval*i, y, *(Mess + i), color);
		}
}






void ScreenSaver(void) {
	uint8_t x, y, i, lighted = 0;

	x = rand() % 63;		// RND_min = 1.00 .
	y = rand() % 31;		// RND_max = 9.99 .

	//  0  1.   50/50  .. = 1.1 .  o_O
	i = rand()/2000000000;

	if(i) lighted++;
	else  lighted--;

	x = x << 1;
	y = y << 1;
	SetPixel(x,   y,   i);
	SetPixel(x+1, y,   i);
	SetPixel(x,   y+1, i);
	SetPixel(x+1, y+1, i);
	//sprintf(sMessage, "%d.", lighted);
	//PutString(0,  49, sMessage, 1);
	UpdateScreen();
}



#endif
